@@ -4,17 +4,18 @@ use cyberdeck::*;
 #[tokio::main]
 async fn main() -> Result<()> {
     let offer = must_read_stdin()?;
-    let mut cd = Cyberdeck::new(
-        |c| {
-            println!("Data channel {} was connected!", c.name());
-            c.send_text("connected!").expect("could not send message");
-        },
-        |c, msg| {
+    let mut cd = Cyberdeck::new(|c, msg| {
+        if let Some(m) = msg {
             println!("Recieved a message from channel {}!", c.name());
-            let msg_str = String::from_utf8(msg.data.to_vec()).unwrap();
-            println!("Message from DataChannel '{}'", msg_str);
-        },
-    )
+            let msg_str = String::from_utf8(m.data.to_vec()).unwrap();
+            println!("Message from DataChannel '{}': {}", c.name(), msg_str);
+        } else if c.state() == RTCDataChannelState::Open {
+            println!("DataChannel '{}' opened", c.name());
+            c.send_text("Connected to client!").unwrap();
+        } else if c.state() == RTCDataChannelState::Closed {
+            println!("DataChannel '{}' closed", c.name());
+        }
+    })
     .await?;
     let answer = cd.connect(offer).await?;
 
